package jp.co.ois.hoge.sample;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.ExitCodeGenerator;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import software.amazon.awssdk.core.ResponseInputStream;
import software.amazon.awssdk.services.dynamodb.DynamoDbClient;
import software.amazon.awssdk.services.dynamodb.model.AttributeValue;
import software.amazon.awssdk.services.dynamodb.model.GetItemRequest;
import software.amazon.awssdk.services.dynamodb.model.GetItemResponse;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.GetObjectRequest;
import software.amazon.awssdk.services.s3.model.GetObjectResponse;

@Slf4j
@Component
public class HogeRunner implements CommandLineRunner, ExitCodeGenerator {

    private int exitCode = 0;

    @Override
    public int getExitCode() {
        return exitCode;
    }

    @Override
    public void run(String... args) throws Exception {
        log.info("start...{}", Arrays.toString(args));
        testMysql();
        testRedis();
        testS3();
        testDynamoDb();
        testWeb();
        exitCode = 0;
        log.info("end...");
    }

    @Autowired
    HogeRepository hogeRepository;

    void testMysql() {
        log.info("called.");
        Map<String, Object> data = hogeRepository.select("1");
        log.info("mysql: {}", data);
    }

    @Autowired
    private StringRedisTemplate redisTemplate;

    void testRedis() {
        log.info("called.");
        redisTemplate.opsForValue().set("ほげ", "あいうえお");
        String value = redisTemplate.opsForValue().get("hoge");
        log.info("redis: {}", value);
    }

    @Autowired
    S3Client s3Client;

    void testS3() {
        log.info("called.");
        ResponseInputStream<GetObjectResponse> res = s3Client.getObject(
                GetObjectRequest.builder().bucket("hoge-s3").key("hoge").build());
        BufferedReader reader = new BufferedReader(new InputStreamReader(res));
        reader.lines().forEach(s -> {
            log.info("s3: {}", s);
        });
    }

    @Autowired
    DynamoDbClient dynamoDbClient;

    void testDynamoDb() {
        log.info("called.");
        Map<String,AttributeValue> keyMap = new HashMap<>();
        keyMap.put("hoge_id", AttributeValue.builder().s("0001").build());
        GetItemResponse res = dynamoDbClient.getItem(
                GetItemRequest.builder().tableName("hoge-dynamo").key(keyMap).build());
        log.info("item: {}", res.item());
    }

    @Autowired
    RestTemplate restTemplate;

    @Data
    @AllArgsConstructor
    static class HogeParam {
        String value;
    }

    void testWeb() {
        log.info("called.");
        String url = "http://localhost/hoge/api";
        HogeParam param = new HogeParam("hoge");
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<HogeParam> request = new HttpEntity<>(param, headers);
        log.info("request: {}", request);
        ResponseEntity<String> res = restTemplate.postForEntity(url, request, String.class);
        log.info("res: {}", res);
    }
}
